import numpy as np


neighbours = [
    [1, 2], [3, 4], [0, 5],
    [1, 5], [0], [4, 2]
]
V = len(neighbours)


def generate_transition_matrix(d):
    P = np.zeros((V, V))
    for v in range(V):
        neigh_count = len(neighbours[v])
        for neigh in neighbours[v]:
            P[v, neigh] = d / neigh_count
        factor = d if neigh_count > 0 else 0
        for u in range(V):
            P[v, u] += (1 - factor) / V
    return P


def get_stationary_distribution(P):
    # Sanity check for rows to sum to 1.
    assert np.allclose(P @ np.ones(V), np.ones(V))

    # Solve these equations simultaneously.
    A = np.concatenate([(P - np.eye(V)).T, np.ones((1, V))])

    return np.linalg.lstsq(A, np.concatenate([np.zeros(V), [1]]))[0]


def run_for(d):
    P = generate_transition_matrix(d)
    pi = get_stationary_distribution(P)
    print(f'The stationary distribution for d={d} is :\n {pi}')


run_for(0.85)
run_for(0.05)
run_for(0.01)

# Expected output;
# The stationary distribution for d=0.85 is :
#  [0.2404889  0.16821698 0.18948795 0.09649221 0.15877238 0.14654157]
# The stationary distribution for d=0.05 is :
#  [0.17083575 0.16666673 0.16676835 0.1625     0.16666413 0.16656504]
# The stationary distribution for d=0.01 is :
#  [0.16750002 0.16666667 0.16667081 0.16583333 0.16666665 0.16666252]
