import numpy as np
import scipy.stats
import matplotlib.pyplot as plt


def generate_matrix(N, r, d):
    P = np.zeros((N + 1, N + 1))
    for x in range(N):
        binom_param = 1 / d
        poisson_param = r * x / d
        for y in range(N):
            # Evaluate P(X_{n+1} = y | X_n = x).
            prob = 0
            for k in range(-min(y - x, 0), x + 1):
                prob += scipy.stats.binom.pmf(k=k, n=x, p=binom_param) * \
                        scipy.stats.poisson.pmf(k=y - (x - k), mu=poisson_param)
            P[x, y] = prob
    # Set the absorbing state.
    P[:N, N] = 1 - np.sum(P[:N, :N], axis=1)
    P[N, N] = 1
    return P


def run_simulation(x0, P, n):
    x = x0
    while n > 0:
        p = P[x]
        x = np.random.choice(a=len(p), p=p)
        n = n - 1
    return x


def calculate_expected(x0, r, d, n):
    return x0 * ((1 + r/d - 1/d) ** n)


r, d = 1.1, 14
N = 200
samples = 1000
x0 = 100

P = generate_matrix(N, r, d)


def plot_for(steps):
    v_samples = [run_simulation(x0, P, steps) for _ in range(samples)]

    plt.hist(v_samples)
    plt.vlines(x=calculate_expected(x0, r, d, steps), ymin=0, ymax=samples/3)
    plt.show()


plot_for(steps=10)
plot_for(steps=20)
plot_for(steps=30)
plot_for(steps=50)
