import numpy as np
import scipy.stats


def generate_matrix(N, r, d):
    P = np.zeros((N + 1, N + 1))
    for x in range(N):
        binom_param = 1 / d
        poisson_param = r * x / d
        for y in range(N):
            # Evaluate P(X_{n+1} = y | X_n = x).
            prob = 0
            for k in range(max(x - y, 0), x + 1):
                prob += scipy.stats.binom.pmf(k=k, n=x, p=binom_param) * \
                        scipy.stats.poisson.pmf(k=y - (x - k), mu=poisson_param)
            P[x, y] = prob
    # Set the absorbing state.
    P[:N, N] = 1 - np.sum(P[:N, :N], axis=1)
    P[N, N] = 1
    return P


r, d = 1.1, 14
N = 100

P = generate_matrix(N, r, d)
# Solve for the hitting probability vector as shown in Section 10.3.
Q = np.copy(P)
Q[0, :] = 0  # The elements of the hitting set have a probability 1.
Q[N, :] = 0  # Absorbing states have 0 probability of reaching the hitting set.
hit_probabilities = np.linalg.solve(np.eye(N + 1) - Q, np.concatenate([np.ones(1), np.zeros(N)]))
print(f"Probability of hitting 0 starting at X50: {hit_probabilities[50]}")
# Expected output: Probability of hitting 0 starting at X50: 0.007035837489220857
