import numpy as np

N = 10  # Number of states for X
total_states = 3 * N


def convert_pair(x, v):
    return v * N + x


def convert_single(a):
    return a % N, a // N


P = np.zeros((total_states, total_states))
for a in range(0, total_states):
    x, v = convert_single(a)
    for dv in range(0, 3):
        pv = 14/15 if v == dv else 1/30
        xx = max(0, min(N - 1, x + v - 1))
        # Different velocities might lead to the same states
        # so we need to accumulate the transition probabilities.
        P[a, convert_pair(xx, dv)] += pv

# Sanity check for rows to sum to 1.
assert np.allclose(P @ np.ones(total_states), np.ones(total_states))

# Solve these equations simultaneously.
A = np.concatenate([(P - np.eye(total_states)).T, np.ones((1, total_states))])
pi = np.linalg.lstsq(A, np.concatenate([np.zeros(total_states), [1]]))[0]
print(f'The stationary distribution is : {pi}')
